package com.jiuqi.util;

import java.text.DecimalFormat;
import java.text.NumberFormat;
import java.util.HashMap;
import java.util.Stack;

public class NumberFormatManager {
   private static HashMap integerFormats = new HashMap();
   private static HashMap numberFormats = new HashMap();
   private static HashMap patternFormats = new HashMap();
   private static final Integer[] INT_VALUES = new Integer[]{new Integer(0), new Integer(1), new Integer(2), new Integer(3), new Integer(4), new Integer(5), new Integer(6), new Integer(7), new Integer(8), new Integer(9), new Integer(10), new Integer(11), new Integer(12), new Integer(13), new Integer(14), new Integer(15), new Integer(16), new Integer(17), new Integer(18), new Integer(19), new Integer(20), new Integer(21), new Integer(22), new Integer(23), new Integer(24), new Integer(25), new Integer(26), new Integer(27), new Integer(28), new Integer(29), new Integer(30), new Integer(31), new Integer(32), new Integer(33), new Integer(34), new Integer(35), new Integer(36), new Integer(37), new Integer(38), new Integer(39)};

   public static NumberFormat createIntegerFormat(int integerDigits) {
      NumberFormat format = null;
      Integer key = getInt(integerDigits);
      synchronized(integerFormats) {
         Stack formats = (Stack)integerFormats.get(key);
         if (formats == null) {
            formats = new Stack();
            integerFormats.put(key, formats);
         }

         if (!formats.isEmpty()) {
            format = (NumberFormat)formats.pop();
         } else {
            format = NumberFormat.getInstance();
            format.setMinimumFractionDigits(0);
            format.setMaximumFractionDigits(0);
            format.setMinimumIntegerDigits(integerDigits);
            format.setGroupingUsed(false);
         }

         return format;
      }
   }

   public static void recycleIntegerFormat(int integerDigits, NumberFormat format) {
      Integer key = getInt(integerDigits);
      synchronized(integerFormats) {
         Stack formats = (Stack)integerFormats.get(key);
         if (formats != null) {
            formats.push(format);
         }

      }
   }

   public static NumberFormat createNumberFormat(int fractionDigits, boolean groupingUsed) {
      return createNumberFormat(fractionDigits, fractionDigits, groupingUsed);
   }

   public static NumberFormat createNumberFormat(int minimumFractionDigits, int maximumFractionDigits, boolean groupingUsed) {
      int keyData = groupingUsed ? 1 : 0;
      keyData |= maximumFractionDigits << 16 | minimumFractionDigits << 8;
      Integer key = new Integer(keyData);
      NumberFormat format = null;
      synchronized(numberFormats) {
         Stack formats = (Stack)numberFormats.get(key);
         if (formats == null) {
            formats = new Stack();
            numberFormats.put(key, formats);
         }

         if (!formats.isEmpty()) {
            format = (NumberFormat)formats.pop();
         } else {
            format = NumberFormat.getInstance();
            format.setMaximumFractionDigits(maximumFractionDigits);
            format.setMinimumFractionDigits(minimumFractionDigits);
            format.setMinimumIntegerDigits(1);
            format.setGroupingUsed(groupingUsed);
         }

         return format;
      }
   }

   public static void recycleNumberFormat(int fractionDigits, boolean groupingUsed, NumberFormat format) {
      recycleNumberFormat(fractionDigits, fractionDigits, groupingUsed, format);
   }

   public static void recycleNumberFormat(int minimumFractionDigits, int maximumFractionDigits, boolean groupingUsed, NumberFormat format) {
      int keyData = groupingUsed ? 1 : 0;
      keyData |= maximumFractionDigits << 16 | minimumFractionDigits << 8;
      Integer key = new Integer(keyData);
      synchronized(numberFormats) {
         Stack formats = (Stack)numberFormats.get(key);
         if (formats != null) {
            formats.push(format);
         }

      }
   }

   public static NumberFormat createNumberFormat(String pattern) {
      NumberFormat format = null;
      synchronized(patternFormats) {
         Stack formats = (Stack)patternFormats.get(pattern);
         if (formats == null) {
            formats = new Stack();
            patternFormats.put(pattern, formats);
         }

         if (!formats.isEmpty()) {
            format = (NumberFormat)formats.pop();
         } else {
            format = new DecimalFormat(pattern);
         }

         return (NumberFormat)format;
      }
   }

   public static void recycleNumberFormat(String pattern, NumberFormat format) {
      synchronized(patternFormats) {
         Stack formats = (Stack)patternFormats.get(pattern);
         if (formats != null) {
            formats.push(format);
         }

      }
   }

   public static final Integer getInt(int index) {
      return index < INT_VALUES.length ? INT_VALUES[index] : new Integer(index);
   }

   public static void main(String[] args) {
      for(int i = 0; i < 2000; ++i) {
         NumberFormatManagerTest1 testor = new NumberFormatManagerTest1();
         testor.start();
      }

   }
}
